/**
 * Copyright (c) 2024 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
#define _HEAVY_CONTEXT_WSTD_SMTHR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvSignalLine.h"
#include "HvControlCast.h"
#include "HvControlBinop.h"
#include "HvControlIf.h"
#include "HvMath.h"
#include "HvSignalEnvelope.h"
#include "HvControlVar.h"
#include "HvSignalVar.h"

class Heavy_WSTD_SMTHR : public HeavyContext {

 public:
  Heavy_WSTD_SMTHR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_SMTHR();

  const char *getName() override { return "WSTD_SMTHR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        LMTR = 0xCB23DC60, // Lmtr
        MIX = 0x97D46AC6, // Mix
        SMTHR = 0xD88E26F9, // Smthr
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_WjpzRfcj_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_ucxR4tWk_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_tQRcu2Dz_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_m9crU8eR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_b565Hb13_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_KnLPLSOd_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_x2ZXPOmS_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_pfKjAsT8_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_4Py6z7UW_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_88nVm36k_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_nGJJSxHz_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_gNamxSYN_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_aBHlFjKJ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_jA0qeKu6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_DdR8BDMj_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_xR802Oct_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_Zjog7L7d_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_bAqX3ni1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_2luOyCFn_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_vvuqfY5k_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_arlUwBBn_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_o5SPXiZi_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_xAOuID5f_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_IpJcE3fb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_iycJI2Ax_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_5bRYvBxX_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_tsdPt2Y7_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_nvePaQKC_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_9cFWRz1q_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_voqlnhqb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_xNmhnLm1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_Vc8Rh0aS_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_7OmSZxTS_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_hFrStpAh_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_KblPt3qs_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_Q9Eub4I2_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_yYAqStGf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_8xw8Ffg3_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_fLhrxHZj_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_zsBU5pKO_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_TzoaJnJf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_cGKJ8lKA_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_RR5vnjCY_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_J7GVvvaS_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_1ulzQDig_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_u8J7YVkk_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_DVHUN4Vg_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_Guc723Fk_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_QQysPIn8_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_WLMAK22e_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_Nt7MDlRQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_nUmaER0k_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_dmm3ZEO3_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Zt8pYaT1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_QnUQ6IpT_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_92AbfFA1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_EqiUJj1P_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_2ReMGTbn_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_7PfQFviq_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_JwzYKogT_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_DDE9XE95_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_Xam0KTwq_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_enxkJDPD;
  SignalEnvelope sEnv_bAqX3ni1;
  SignalEnvelope sEnv_2luOyCFn;
  SignalEnvelope sEnv_KblPt3qs;
  SignalEnvelope sEnv_Q9Eub4I2;
  SignalLine sLine_b42gfyKD;
  SignalLine sLine_i17Bnk5J;
  ControlBinop cBinop_WjpzRfcj;
  ControlBinop cBinop_ucxR4tWk;
  ControlBinop cBinop_tQRcu2Dz;
  ControlBinop cBinop_m9crU8eR;
  ControlBinop cBinop_x2ZXPOmS;
  ControlBinop cBinop_4Py6z7UW;
  ControlBinop cBinop_88nVm36k;
  ControlBinop cBinop_nGJJSxHz;
  ControlVar cVar_gNamxSYN;
  ControlVar cVar_jA0qeKu6;
  SignalVarf sVarf_G7LvcIG9;
  ControlVar cVar_DdR8BDMj;
  ControlIf cIf_xR802Oct;
  ControlIf cIf_Zjog7L7d;
  ControlBinop cBinop_vvuqfY5k;
  ControlBinop cBinop_o5SPXiZi;
  ControlBinop cBinop_xAOuID5f;
  ControlBinop cBinop_5bRYvBxX;
  ControlBinop cBinop_nvePaQKC;
  SignalVarf sVarf_7vm1MmwC;
  ControlVar cVar_Vc8Rh0aS;
  ControlIf cIf_7OmSZxTS;
  ControlIf cIf_hFrStpAh;
  ControlBinop cBinop_yYAqStGf;
  ControlBinop cBinop_fLhrxHZj;
  ControlBinop cBinop_zsBU5pKO;
  ControlBinop cBinop_RR5vnjCY;
  ControlBinop cBinop_1ulzQDig;
  ControlVar cVar_QQysPIn8;
  ControlBinop cBinop_WLMAK22e;
  SignalVarf sVarf_PPbyL15Z;
  SignalVarf sVarf_jIeQyvIt;
  SignalVarf sVarf_kSQxQboE;
  SignalVarf sVarf_ayIg1vFD;
  ControlBinop cBinop_Zt8pYaT1;
};

#endif // _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
